#include <SDL/SDL.h>
#include "dagger.h"
#include "sdlhandler.h"

#define TRUE	1
#define FALSE 	0

// The nulldriver keyboard handler is gkeyboard...
extern void gKeyboard(int key, int state) ;

typedef unsigned short  color;

struct {
  int 		width, height;
  Uint32 	palette[256];
  
} gScreenData;
static Dagger *		gpDagger;
static SDL_Surface *	gpScreen;
static int 		gbInRender = 0;



int sdlWrapperInit(Uint32 w, Uint32 h) {
  int i;
  atexit(SDL_Quit);

    if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER) != 0) {
	printf("Unable to initialize SDL: %s\n", SDL_GetError());
	return FALSE;
  }
    
  gpScreen = SDL_SetVideoMode(w, h, 16, SDL_DOUBLEBUF);
  if (gpScreen == NULL) {
	  printf("Unable to set video mode: %s\n", SDL_GetError());
	  return FALSE;
  }
  
  gScreenData.width = w;
  gScreenData.height = h;

  // Default palette - pretty poor, TBH
  for(i=0;i<256;++i) {
    gScreenData.palette[i] = (i<<16) | (i<<8) | i;
  }

  gScreenData.palette[34] = 0xff0000;
  gScreenData.palette[85] = 0x00ff00;
  gScreenData.palette[223]= 0x0000ff;
  gScreenData.palette[238]= 0xffff00;
  gScreenData.palette[17] = 0xffffff;
  gScreenData.palette[63] = 0x777777;
  gScreenData.palette[221]= 0x008000;

  gScreenData.palette[1]= 0xffffff;
  gScreenData.palette[255]= 0x000000;

  return TRUE;
}


// Should be an RGBA palette of 256 entries
void
sdlWrapperSetPalette(const Uint32 *pPalette) {
  int i;
  
  for(i=0;i<256;++i) {
    gScreenData.palette[i] = pPalette[i];
  }
}


void sdlWrapperPreRender() {
  SDL_LockSurface(gpScreen);
  
  gbInRender = 1;
}

void sdlWrapperPostRender() {
  SDL_UnlockSurface(gpScreen);
  SDL_Flip(gpScreen);
  
  gbInRender = 0;
}



void
sdlDrawPixel1(unsigned int offset, color cl) {
}

void
sdlDrawPixel2(unsigned int offset, color cl) {
  // BUGWARN: There's a ASPECT_XCORRECTION in offset calculation:
  //     register unsigned int offset = y * camera_screen.buffer_width + ASPECT_XCORRECTION(x);
  int x = offset % gScreenData.width;
  int y = offset / gScreenData.width;
  Uint32 rgbColor =  gScreenData.palette[cl&0x0ff];  
  int resetRender = 0;
  
  if (!gbInRender) {
    sdlWrapperPreRender();
    resetRender = 1;
  }
     
  Uint32 colour16 = SDL_MapRGB( gpScreen->format, (rgbColor&0xff0000)>>16,  (rgbColor&0xff00)>>8, (rgbColor&0xff)>>0 );    
  Uint8 *ptr = (Uint8 *)gpScreen->pixels;
  ptr += x * 2;	// 2 because we're dealing with 16 bit pixels
  ptr += y * gpScreen->pitch;

  *(Uint16 *)ptr = colour16;
   
  if (resetRender) {
    sdlWrapperPostRender();
  }
   
}


void
sdlDrawPixel3(unsigned int offset, color cl) {
}

void
sdlDrawPixel4(unsigned int offset, color cl) {
}



static Uint32 updateLoop(Uint32 interval, void *param) {
    SDL_Event event;
    SDL_UserEvent userevent;

    /* In this example, our callback pushes an SDL_USEREVENT event
    into the queue, and causes our callback to be called again at the
    same interval: */

    userevent.type = SDL_USEREVENT;
    userevent.code = 0;
    userevent.data1 = NULL;
    userevent.data2 = NULL;

    event.type = SDL_USEREVENT;
    event.user = userevent;

    SDL_PushEvent(&event);
    return(interval);
}


void sdlWrapperEventLoop() {

  SDL_TimerID timerID = SDL_AddTimer(1000/30, updateLoop, 0);

  SDL_Event event;
  int quit = 0;

  while (!quit)  {
    while(SDL_PollEvent(&event))  {
	switch(event.type) {
	  case SDL_QUIT:
	    quit = 1;
	    break;
	  
	  case SDL_USEREVENT:
	    daggerUpdate(gpDagger);
	    break;
	    
	  case SDL_KEYDOWN:
	    gKeyboard(event.key.keysym.sym, 1);
	    break;
	    
	  case SDL_KEYUP:
	    gKeyboard(event.key.keysym.sym, 0);
	    break;

	  default:
	    break;
	}
       }
    }
    
    SDL_RemoveTimer(timerID);
}


void 
sdlWrapperReady(Dagger *pDagger) {  
  gpDagger = pDagger;
  //
  int bpp = gpScreen->format->BytesPerPixel; 
  switch(bpp) {
    case 1:
      draw_set_draw_proc(sdlDrawPixel1);
      break;
    case 2:
      draw_set_draw_proc(sdlDrawPixel2);
      break;
    case 3:
      draw_set_draw_proc(sdlDrawPixel3);
      break;
    case 4:
      draw_set_draw_proc(sdlDrawPixel4);
      break;
  }
}

void 
sdlWrapperExit() {  
  
  SDL_FreeSurface(gpScreen);
  gpScreen = NULL;
}
